import json
from typing import *

import httpx

from ..api_config import APIConfig, HTTPException
from ..models import *


def healthcheck_v0_health_get(api_config_override: Optional[APIConfig] = None) -> Any:
    api_config = api_config_override if api_config_override else APIConfig()

    base_path = api_config.base_path
    path = f"/v0/health"
    headers = {
        "Content-Type": "application/json",
        "Accept": "application/json",
        "Authorization": f"Bearer { api_config.get_access_token() }",
    }
    query_params: Dict[str, Any] = {}

    query_params = {key: value for (key, value) in query_params.items() if value is not None}

    with httpx.Client(base_url=base_path, verify=api_config.verify) as client:
        response = client.request(
            "get",
            httpx.URL(path),
            headers=headers,
            params=query_params,
        )

    if response.status_code != 200:
        raise HTTPException(response.status_code, f" failed with status code: {response.status_code}")

    return response.json()


def create_project_v0_project_post(
    api_key: str, data: ProjectSchema, api_config_override: Optional[APIConfig] = None
) -> ProjectResponse:
    api_config = api_config_override if api_config_override else APIConfig()

    base_path = api_config.base_path
    path = f"/v0/project"
    headers = {
        "Content-Type": "application/json",
        "Accept": "application/json",
        "Authorization": f"Bearer { api_config.get_access_token() }",
        "api-key": api_key,
    }
    query_params: Dict[str, Any] = {}

    query_params = {key: value for (key, value) in query_params.items() if value is not None}

    with httpx.Client(base_url=base_path, verify=api_config.verify) as client:
        response = client.request("post", httpx.URL(path), headers=headers, params=query_params, json=data.dict())

    if response.status_code != 201:
        raise HTTPException(response.status_code, f" failed with status code: {response.status_code}")

    return ProjectResponse(**response.json()) if response.json() is not None else ProjectResponse()


def get_project_v0_project__project_id__get(
    project_id: int, api_key: str, api_config_override: Optional[APIConfig] = None
) -> ProjectResponse:
    api_config = api_config_override if api_config_override else APIConfig()

    base_path = api_config.base_path
    path = f"/v0/project/{project_id}"
    headers = {
        "Content-Type": "application/json",
        "Accept": "application/json",
        "Authorization": f"Bearer { api_config.get_access_token() }",
        "api-key": api_key,
    }
    query_params: Dict[str, Any] = {}

    query_params = {key: value for (key, value) in query_params.items() if value is not None}

    with httpx.Client(base_url=base_path, verify=api_config.verify) as client:
        response = client.request(
            "get",
            httpx.URL(path),
            headers=headers,
            params=query_params,
        )

    if response.status_code != 201:
        raise HTTPException(response.status_code, f" failed with status code: {response.status_code}")

    return ProjectResponse(**response.json()) if response.json() is not None else ProjectResponse()


def get_project_v0_project__project_id__put(
    project_id: int, api_key: str, data: ProjectSchema, api_config_override: Optional[APIConfig] = None
) -> ProjectResponse:
    api_config = api_config_override if api_config_override else APIConfig()

    base_path = api_config.base_path
    path = f"/v0/project/{project_id}"
    headers = {
        "Content-Type": "application/json",
        "Accept": "application/json",
        "Authorization": f"Bearer { api_config.get_access_token() }",
        "api-key": api_key,
    }
    query_params: Dict[str, Any] = {}

    query_params = {key: value for (key, value) in query_params.items() if value is not None}

    with httpx.Client(base_url=base_path, verify=api_config.verify) as client:
        response = client.request("put", httpx.URL(path), headers=headers, params=query_params, json=data.dict())

    if response.status_code != 201:
        raise HTTPException(response.status_code, f" failed with status code: {response.status_code}")

    return ProjectResponse(**response.json()) if response.json() is not None else ProjectResponse()


def get_generations_v0_generations_get(
    api_key: str, api_config_override: Optional[APIConfig] = None
) -> List[GenerationList]:
    api_config = api_config_override if api_config_override else APIConfig()

    base_path = api_config.base_path
    path = f"/v0/generations"
    headers = {
        "Content-Type": "application/json",
        "Accept": "application/json",
        "Authorization": f"Bearer { api_config.get_access_token() }",
        "api-key": api_key,
    }
    query_params: Dict[str, Any] = {}

    query_params = {key: value for (key, value) in query_params.items() if value is not None}

    with httpx.Client(base_url=base_path, verify=api_config.verify) as client:
        response = client.request(
            "get",
            httpx.URL(path),
            headers=headers,
            params=query_params,
        )

    if response.status_code != 200:
        raise HTTPException(response.status_code, f" failed with status code: {response.status_code}")

    return [GenerationList(**item) for item in response.json()]


def add_generation_v0_generations_post(
    api_key: str, data: GenerationPayload, api_config_override: Optional[APIConfig] = None
) -> GenerationResponse:
    api_config = api_config_override if api_config_override else APIConfig()

    base_path = api_config.base_path
    path = f"/v0/generations"
    headers = {
        "Content-Type": "application/json",
        "Accept": "application/json",
        "Authorization": f"Bearer { api_config.get_access_token() }",
        "api-key": api_key,
    }
    query_params: Dict[str, Any] = {}

    query_params = {key: value for (key, value) in query_params.items() if value is not None}

    with httpx.Client(base_url=base_path, verify=api_config.verify) as client:
        response = client.request("post", httpx.URL(path), headers=headers, params=query_params, json=data.dict())

    if response.status_code != 201:
        raise HTTPException(response.status_code, f" failed with status code: {response.status_code}")

    return GenerationResponse(**response.json()) if response.json() is not None else GenerationResponse()


def get_generation_v0_generations__generation_id__get(
    generation_id: str, api_key: str, api_config_override: Optional[APIConfig] = None
) -> str:
    api_config = api_config_override if api_config_override else APIConfig()

    base_path = api_config.base_path
    path = f"/v0/generations/{generation_id}"
    headers = {
        "Content-Type": "application/json",
        "Accept": "application/json",
        "Authorization": f"Bearer { api_config.get_access_token() }",
        "api-key": api_key,
    }
    query_params: Dict[str, Any] = {}

    query_params = {key: value for (key, value) in query_params.items() if value is not None}

    with httpx.Client(base_url=base_path, verify=api_config.verify) as client:
        response = client.request(
            "get",
            httpx.URL(path),
            headers=headers,
            params=query_params,
        )

    if response.status_code != 200:
        raise HTTPException(response.status_code, f" failed with status code: {response.status_code}")

    return response.json()


def delete_generation_v0_generations__generation_id__delete(
    generation_id: str, api_key: str, api_config_override: Optional[APIConfig] = None
) -> None:
    api_config = api_config_override if api_config_override else APIConfig()

    base_path = api_config.base_path
    path = f"/v0/generations/{generation_id}"
    headers = {
        "Content-Type": "application/json",
        "Accept": "application/json",
        "Authorization": f"Bearer { api_config.get_access_token() }",
        "api-key": api_key,
    }
    query_params: Dict[str, Any] = {}

    query_params = {key: value for (key, value) in query_params.items() if value is not None}

    with httpx.Client(base_url=base_path, verify=api_config.verify) as client:
        response = client.request(
            "delete",
            httpx.URL(path),
            headers=headers,
            params=query_params,
        )

    if response.status_code != 204:
        raise HTTPException(response.status_code, f" failed with status code: {response.status_code}")

    return None


def get_generation_schema_v0_generations__generation_id__schema_get(
    generation_id: str, api_key: str, api_config_override: Optional[APIConfig] = None
) -> str:
    api_config = api_config_override if api_config_override else APIConfig()

    base_path = api_config.base_path
    path = f"/v0/generations/{generation_id}/schema"
    headers = {
        "Content-Type": "application/json",
        "Accept": "application/json",
        "Authorization": f"Bearer { api_config.get_access_token() }",
        "api-key": api_key,
    }
    query_params: Dict[str, Any] = {}

    query_params = {key: value for (key, value) in query_params.items() if value is not None}

    with httpx.Client(base_url=base_path, verify=api_config.verify) as client:
        response = client.request(
            "get",
            httpx.URL(path),
            headers=headers,
            params=query_params,
        )

    if response.status_code != 200:
        raise HTTPException(response.status_code, f" failed with status code: {response.status_code}")

    return response.json()


def get_generation_view_v0_generations__generation_id__view_get(
    generation_id: str, api_key: str, api_config_override: Optional[APIConfig] = None
) -> str:
    api_config = api_config_override if api_config_override else APIConfig()

    base_path = api_config.base_path
    path = f"/v0/generations/{generation_id}/view"
    headers = {
        "Content-Type": "application/json",
        "Accept": "application/json",
        "Authorization": f"Bearer { api_config.get_access_token() }",
        "api-key": api_key,
    }
    query_params: Dict[str, Any] = {}

    query_params = {key: value for (key, value) in query_params.items() if value is not None}

    with httpx.Client(base_url=base_path, verify=api_config.verify) as client:
        response = client.request(
            "get",
            httpx.URL(path),
            headers=headers,
            params=query_params,
        )

    if response.status_code != 200:
        raise HTTPException(response.status_code, f" failed with status code: {response.status_code}")

    return response.json()


def get_datapoints_v0_find_datapoints_post(
    api_key: str, data: DatapointSearch, api_config_override: Optional[APIConfig] = None
) -> List[DatapointList]:
    api_config = api_config_override if api_config_override else APIConfig()

    base_path = api_config.base_path
    path = f"/v0/find_datapoints"
    headers = {
        "Content-Type": "application/json",
        "Accept": "application/json",
        "Authorization": f"Bearer { api_config.get_access_token() }",
        "api-key": api_key,
    }
    query_params: Dict[str, Any] = {}

    query_params = {key: value for (key, value) in query_params.items() if value is not None}

    with httpx.Client(base_url=base_path, verify=api_config.verify) as client:
        response = client.request("post", httpx.URL(path), headers=headers, params=query_params, json=data.dict())

    if response.status_code != 200:
        raise HTTPException(response.status_code, f" failed with status code: {response.status_code}")

    return [DatapointList(**item) for item in response.json()]


def add_datapoint_v0_datapoints_post(
    api_key: str, data: DatapointSchema, api_config_override: Optional[APIConfig] = None
) -> DatapointResponse:
    api_config = api_config_override if api_config_override else APIConfig()

    base_path = api_config.base_path
    path = f"/v0/datapoints"
    headers = {
        "Content-Type": "application/json",
        "Accept": "application/json",
        "Authorization": f"Bearer { api_config.get_access_token() }",
        "api-key": api_key,
    }
    query_params: Dict[str, Any] = {}

    query_params = {key: value for (key, value) in query_params.items() if value is not None}

    with httpx.Client(base_url=base_path, verify=api_config.verify) as client:
        response = client.request("post", httpx.URL(path), headers=headers, params=query_params, json=data.dict())

    if response.status_code != 201:
        raise HTTPException(response.status_code, f" failed with status code: {response.status_code}")

    return DatapointResponse(**response.json()) if response.json() is not None else DatapointResponse()


def register_model_v0_register_model_post(
    api_key: str, data: ModelUnderTestSchema, api_config_override: Optional[APIConfig] = None
) -> ModelUnderTestResponse:
    api_config = api_config_override if api_config_override else APIConfig()

    base_path = api_config.base_path
    path = f"/v0/register_model"
    headers = {
        "Content-Type": "application/json",
        "Accept": "application/json",
        "Authorization": f"Bearer { api_config.get_access_token() }",
        "api-key": api_key,
    }
    query_params: Dict[str, Any] = {}

    query_params = {key: value for (key, value) in query_params.items() if value is not None}

    with httpx.Client(base_url=base_path, verify=api_config.verify) as client:
        response = client.request("post", httpx.URL(path), headers=headers, params=query_params, json=data.dict())

    if response.status_code != 201:
        raise HTTPException(response.status_code, f" failed with status code: {response.status_code}")

    return ModelUnderTestResponse(**response.json()) if response.json() is not None else ModelUnderTestResponse()



from .DatapointList import *
from .DatapointResponse import *
from .DatapointSchema import *
from .DatapointSearch import *
from .GenerationList import *
from .GenerationPayload import *
from .GenerationResponse import *
from .HTTPValidationError import *
from .ModelUnderTestResponse import *
from .ModelUnderTestSchema import *
from .ProjectResponse import *
from .ProjectSchema import *
from .ValidationError import *



from typing import *

from pydantic import BaseModel, Field


class DatapointSchema(BaseModel):
    """
    DatapointSchema model

    """

    tags: Optional[List[str]] = Field(alias="tags", default=None)

    input: str = Field(alias="input")

    input_datetime: Optional[Union[str, Any]] = Field(alias="input_datetime", default=None)

    result: str = Field(alias="result")

    result_datetime: Optional[Union[str, Any]] = Field(alias="result_datetime", default=None)

    feedback: Optional[Union[int, Any]] = Field(alias="feedback", default=None)

    error_message: Optional[Union[str, Any]] = Field(alias="error_message", default=None)

    error_code: Optional[Union[str, Any]] = Field(alias="error_code", default=None)

    context_token: Optional[Union[str, Any]] = Field(alias="context_token", default=None)

    project_id: Optional[Union[int, Any]] = Field(alias="project_id", default=None)

    mut_id: Optional[int] = Field(alias="mut_id", default=None)



from typing import *

from pydantic import BaseModel, Field


class ModelUnderTestResponse(BaseModel):
    """
    ModelUnderTestResponse model

    """

    id: int = Field(alias="id")

    project_id: int = Field(alias="project_id")

    name: str = Field(alias="name")

    tags: List[str] = Field(alias="tags")



from typing import *

from pydantic import BaseModel, Field


class ProjectResponse(BaseModel):
    """
    ProjectResponse model

    """

    id: int = Field(alias="id")

    name: str = Field(alias="name")

    tags: Union[List[str], Any] = Field(alias="tags")



from typing import *

from pydantic import BaseModel, Field


class ModelUnderTestSchema(BaseModel):
    """
    ModelUnderTestSchema model

    """

    name: Optional[str] = Field(alias="name", default=None)

    tags: Optional[Union[List[str], Any]] = Field(alias="tags", default=None)

    project_id: Optional[Union[int, Any]] = Field(alias="project_id", default=None)



from typing import *

from pydantic import BaseModel, Field


class DatapointList(BaseModel):
    """
    DatapointList model

    """

    id: int = Field(alias="id")

    tags: List[str] = Field(alias="tags")

    feedback: Union[int, Any] = Field(alias="feedback")

    error_message: Union[str, Any] = Field(alias="error_message")

    error_code: Union[str, Any] = Field(alias="error_code")

    time_created: str = Field(alias="time_created")



import json
from datetime import datetime
from typing import List, Union

from okareo_api_client.api_config import HTTPException
from okareo_api_client.models import (
    DatapointSchema,
    GenerationList,
    ModelUnderTestResponse,
    ModelUnderTestSchema,
)
from okareo_api_client.services.None_service import (
    add_datapoint_v0_datapoints_post,
    get_generations_v0_generations_get,
    register_model_v0_register_model_post,
)


class Okareo:
    """A class for interacting with Okareo API"""

    def __init__(self, api_key: str):
        self.api_key = api_key

    def get_generations(self) -> Union[List[GenerationList], None]:
        """Get a list of generations"""
        data = get_generations_v0_generations_get(self.api_key)
        if isinstance(data, HTTPException):
            print(f"Unexpected {data=}, {type(data)=}")
            raise
        return data

    def register_model(self) -> ModelUnderTestResponse:
        data = ModelUnderTestSchema(name="test-model", tags=["aaa", "bbb", "ccc"])
        return register_model_v0_register_model_post(self.api_key, data)

    def add_data_point(
        self,
        input_: Union[dict, str],
        result: Union[dict, str],
        feedback: int,
        context_token: str,
        mut_id: int,
        error_message: Union[str, None] = None,
        error_code: Union[str, None] = None,
        input_datetime: Union[datetime, None] = None,
        result_datetime: Union[datetime, None] = None,
        project_id: Union[int, None] = None,
        tags: Union[List[str], None] = None,
    ) -> None:
        body = {
            "tags": tags or [],
            "input": json.dumps(input_),
            "result": json.dumps(result),
            "context_token": context_token,
            "feedback": feedback,
            "error_message": error_message,
            "error_code": error_code,
            "input_datetime": input_datetime,
            "result_datetime": result_datetime,
            "project_id": project_id,
            "mut_id": mut_id,
        }
        add_datapoint_v0_datapoints_post(
            self.api_key,
            DatapointSchema.model_validate(body, strict=True),
        )



import os
from typing import Optional, Union

from pydantic import BaseModel, Field


class APIConfig(BaseModel):
    base_path: str = "https://api.okareo.com/"
    verify: Union[bool, str] = True

    def get_access_token(self) -> Optional[str]:
        try:
            return os.environ["access_token"]
        except KeyError:
            return None

    def set_access_token(self, value: str):
        raise Exception(
            "This client was generated with an environment variable for the access token. Please set the environment variable 'access_token' to the access token."
        )


class HTTPException(Exception):
    def __init__(self, status_code: int, message: str):
        self.status_code = status_code
        self.message = message
        super().__init__(f"{status_code} {message}")

    def __str__(self):
        return f"{self.status_code} {self.message}"



from typing import *

from pydantic import BaseModel, Field


class DatapointSearch(BaseModel):
    """
    DatapointSearch model

    """

    tags: Optional[List[str]] = Field(alias="tags", default=None)

    from_date: Optional[Union[str, Any]] = Field(alias="from_date", default=None)

    to_date: Optional[Union[str, Any]] = Field(alias="to_date", default=None)

    feedback: Optional[Union[int, Any]] = Field(alias="feedback", default=None)

    error_code: Optional[Union[str, Any]] = Field(alias="error_code", default=None)

    context_token: Optional[Union[str, Any]] = Field(alias="context_token", default=None)

    project_id: Optional[Union[int, Any]] = Field(alias="project_id", default=None)

    mut_id: Optional[Union[int, Any]] = Field(alias="mut_id", default=None)



from typing import *

from pydantic import BaseModel, Field

from .ValidationError import ValidationError


class HTTPValidationError(BaseModel):
    """
    HTTPValidationError model

    """

    detail: Optional[List[Optional[ValidationError]]] = Field(alias="detail", default=None)



from typing import *

from pydantic import BaseModel, Field


class GenerationList(BaseModel):
    """
    GenerationList model

    """

    hash: str = Field(alias="hash")

    time_created: str = Field(alias="time_created")



from typing import *

from pydantic import BaseModel, Field


class DatapointResponse(BaseModel):
    """
    DatapointResponse model

    """

    id: int = Field(alias="id")

    project_id: Optional[Union[int, Any]] = Field(alias="project_id", default=None)

    mut_id: Optional[int] = Field(alias="mut_id", default=None)



from typing import *

from pydantic import BaseModel, Field


class ValidationError(BaseModel):
    """
    ValidationError model

    """

    loc: List[Union[str, int]] = Field(alias="loc")

    msg: str = Field(alias="msg")

    type: str = Field(alias="type")



from typing import *

from pydantic import BaseModel, Field


class GenerationPayload(BaseModel):
    """
    GenerationPayload model

    """

    schema: str = Field(alias="schema")

    view: str = Field(alias="view")

    return_results: Optional[bool] = Field(alias="return_results", default=None)



from typing import *

from pydantic import BaseModel, Field


class GenerationResponse(BaseModel):
    """
    GenerationResponse model

    """

    hash: str = Field(alias="hash")

    data: str = Field(alias="data")
